"""Implementation of statechart statechart.
Generated by itemis CREATE code generator.
"""

import queue
import sys, os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src')))
from yakindu.rx import Observable

class Statechart:
	"""Implementation of the state machine Statechart.
	"""

	class State:
		""" State Enum
		"""
		(
			main_region_magnetron_off,
			main_region_magnetron_on,
			null_state
		) = range(3)
	
	
	def __init__(self):
		""" Declares all necessary variables including list of states, histories etc. 
		"""
		
		self.start_pressed = None
		self.stop_pressed = None
		self.increase_time_pressed = None
		self.door_opened = None
		self.door_closed = None
		self.turn_magnetron_on = None
		self.turn_magnetron_on_observable = Observable()
		self.turn_magnetron_off = None
		self.turn_magnetron_off_observable = Observable()
		self.set_displayed_time = None
		self.set_displayed_time_value = None
		self.set_displayed_time_observable = Observable()
		self.ring_bell = None
		self.ring_bell_observable = Observable()
		
		self.in_event_queue = queue.Queue()
		# enumeration of all states:
		self.__State = Statechart.State
		self.__state_conf_vector_changed = None
		self.__state_vector = [None] * 1
		for __state_index in range(1):
			self.__state_vector[__state_index] = self.State.null_state
		
		# initializations:
		self.__is_executing = False
	
	def is_active(self):
		"""Checks if the state machine is active.
		"""
		return self.__state_vector[0] is not self.__State.null_state
	
	def is_final(self):
		"""Checks if the statemachine is final.
		Always returns 'false' since this state machine can never become final.
		"""
		return False
			
	def is_state_active(self, state):
		"""Checks if the state is currently active.
		"""
		s = state
		if s == self.__State.main_region_magnetron_off:
			return self.__state_vector[0] == self.__State.main_region_magnetron_off
		if s == self.__State.main_region_magnetron_on:
			return self.__state_vector[0] == self.__State.main_region_magnetron_on
		return False
		
	def __execute_queued_event(self, func):
		func()
	
	def __get_next_event(self):
		if not self.in_event_queue.empty():
			return self.in_event_queue.get()
		return None
	
	def raise_start_pressed(self):
		"""Raise method for event start_pressed.
		"""
		self.in_event_queue.put(self.__raise_start_pressed_call)
		self.run_cycle()
	
	def __raise_start_pressed_call(self):
		"""Raise callback for event start_pressed.
		"""
		self.start_pressed = True
	
	def raise_stop_pressed(self):
		"""Raise method for event stop_pressed.
		"""
		self.in_event_queue.put(self.__raise_stop_pressed_call)
		self.run_cycle()
	
	def __raise_stop_pressed_call(self):
		"""Raise callback for event stop_pressed.
		"""
		self.stop_pressed = True
	
	def raise_increase_time_pressed(self):
		"""Raise method for event increase_time_pressed.
		"""
		self.in_event_queue.put(self.__raise_increase_time_pressed_call)
		self.run_cycle()
	
	def __raise_increase_time_pressed_call(self):
		"""Raise callback for event increase_time_pressed.
		"""
		self.increase_time_pressed = True
	
	def raise_door_opened(self):
		"""Raise method for event door_opened.
		"""
		self.in_event_queue.put(self.__raise_door_opened_call)
		self.run_cycle()
	
	def __raise_door_opened_call(self):
		"""Raise callback for event door_opened.
		"""
		self.door_opened = True
	
	def raise_door_closed(self):
		"""Raise method for event door_closed.
		"""
		self.in_event_queue.put(self.__raise_door_closed_call)
		self.run_cycle()
	
	def __raise_door_closed_call(self):
		"""Raise callback for event door_closed.
		"""
		self.door_closed = True
	
	def __enter_sequence_main_region_magnetron_off_default(self):
		"""'default' enter sequence for state MagnetronOff.
		"""
		#'default' enter sequence for state MagnetronOff
		self.__state_vector[0] = self.State.main_region_magnetron_off
		self.__state_conf_vector_changed = True
		
	def __enter_sequence_main_region_magnetron_on_default(self):
		"""'default' enter sequence for state MagnetronOn.
		"""
		#'default' enter sequence for state MagnetronOn
		self.__state_vector[0] = self.State.main_region_magnetron_on
		self.__state_conf_vector_changed = True
		
	def __enter_sequence_main_region_default(self):
		"""'default' enter sequence for region main region.
		"""
		#'default' enter sequence for region main region
		self.__react_main_region__entry_default()
		
	def __exit_sequence_main_region_magnetron_off(self):
		"""Default exit sequence for state MagnetronOff.
		"""
		#Default exit sequence for state MagnetronOff
		self.__state_vector[0] = self.State.null_state
		
	def __exit_sequence_main_region_magnetron_on(self):
		"""Default exit sequence for state MagnetronOn.
		"""
		#Default exit sequence for state MagnetronOn
		self.__state_vector[0] = self.State.null_state
		
	def __exit_sequence_main_region(self):
		"""Default exit sequence for region main region.
		"""
		#Default exit sequence for region main region
		state = self.__state_vector[0]
		if state == self.State.main_region_magnetron_off:
			self.__exit_sequence_main_region_magnetron_off()
		elif state == self.State.main_region_magnetron_on:
			self.__exit_sequence_main_region_magnetron_on()
		
	def __react_main_region__entry_default(self):
		"""Default react sequence for initial entry .
		"""
		#Default react sequence for initial entry 
		self.__enter_sequence_main_region_magnetron_off_default()
		
	def __react(self, transitioned_before):
		"""Implementation of __react function.
		"""
		#State machine reactions.
		return transitioned_before
	
	
	def __main_region_magnetron_off_react(self, transitioned_before):
		"""Implementation of __main_region_magnetron_off_react function.
		"""
		#The reactions of state MagnetronOff.
		transitioned_after = self.__react(transitioned_before)
		if transitioned_after < 0:
			if self.start_pressed:
				self.__exit_sequence_main_region_magnetron_off()
				self.turn_magnetron_on_observable.next()
				self.__enter_sequence_main_region_magnetron_on_default()
				transitioned_after = 0
		return transitioned_after
	
	
	def __main_region_magnetron_on_react(self, transitioned_before):
		"""Implementation of __main_region_magnetron_on_react function.
		"""
		#The reactions of state MagnetronOn.
		transitioned_after = self.__react(transitioned_before)
		if transitioned_after < 0:
			if self.stop_pressed:
				self.__exit_sequence_main_region_magnetron_on()
				self.turn_magnetron_off_observable.next()
				self.__enter_sequence_main_region_magnetron_off_default()
				transitioned_after = 0
		return transitioned_after
	
	
	def __clear_in_events(self):
		"""Implementation of __clear_in_events function.
		"""
		self.start_pressed = False
		self.stop_pressed = False
		self.increase_time_pressed = False
		self.door_opened = False
		self.door_closed = False
	
	
	def __micro_step(self):
		"""Implementation of __micro_step function.
		"""
		state = self.__state_vector[0]
		if state == self.State.main_region_magnetron_off:
			self.__main_region_magnetron_off_react(-1)
		elif state == self.State.main_region_magnetron_on:
			self.__main_region_magnetron_on_react(-1)
	
	
	def run_cycle(self):
		"""Implementation of run_cycle function.
		"""
		#Performs a 'run to completion' step.
		if self.__is_executing:
			return
		self.__is_executing = True
		next_event = self.__get_next_event()
		if next_event is not None:
			self.__execute_queued_event(next_event)
		condition_0 = True
		while condition_0:
			self.__micro_step()
			self.__clear_in_events()
			condition_0 = False
			next_event = self.__get_next_event()
			if next_event is not None:
				self.__execute_queued_event(next_event)
				condition_0 = True
		self.__is_executing = False
	
	
	def enter(self):
		"""Implementation of enter function.
		"""
		#Activates the state machine.
		if self.__is_executing:
			return
		self.__is_executing = True
		#Default enter sequence for statechart Statechart
		self.__enter_sequence_main_region_default()
		self.__is_executing = False
	
	
	def exit(self):
		"""Implementation of exit function.
		"""
		#Deactivates the state machine.
		if self.__is_executing:
			return
		self.__is_executing = True
		#Default exit sequence for statechart Statechart
		self.__exit_sequence_main_region()
		self.__is_executing = False
	
	
	def trigger_without_event(self):
		"""Implementation of triggerWithoutEvent function.
		"""
		self.run_cycle()
	
